/*
*    check_snmp_bandwidth . A plugin to measure Bandwidth via SNMP
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; see the file COPYING. If not, write to the
*    Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <sys/types.h>
#include <stdbool.h>
#include <string.h>
#include <mysql/mysql.h>

#define OK 0
#define UNKNOWN 3

// https://www.tutorialspoint.com/data_structures_algorithms/linked_list_program_in_c.htm
struct node {
  char *id;           // The objects ID number
  char *secondhost;   // does the object have a second host
  char *hostgroup;    // does the object have a hostgroup
  struct node *next;
};

struct hostnode {
  char *id;         // The objects ID number
  char *hostname;   // does the object have a second host
  struct hostnode *next;
};

// Create the Service Struct
struct node* service = NULL;
// Create the hostgroup Struct
struct node* hostgroup = NULL;
// Create the host Struct
struct hostnode *host = NULL;

//https://www.hackerearth.com/practice/data-structures/linked-list/singly-linked-list/tutorial/
//http://www.zentut.com/c-tutorial/c-linked-list/#C_Linked_List_implementation

bool debug = false;
bool singlehost = false;
bool multiplehost = false;
bool donotrestoreconfigs = false;
char *username;
char *hostid;
char *multihostname;
char *password;
char *mysqlhost;
char *mysqldatabase;

MYSQL *conn;

char *gethostname( char *hostname );
int getservicename( struct hostnode *head );
int gethostinhostgroup( struct hostnode *head );
int getserviceinhostgroup( struct node *head );
int getserviceinsecondhost( struct node *head );
int write_to_database( );
int is_integer( char *number );
void print_version( void );
void usage( void );
void printList( struct node *head );
void printhostList( struct hostnode *head );
void deleteKey( struct hostnode **head_ref );
void insertFirst( char *id, struct node **head, bool initialize );
void insertFirsthost( char *hostname, struct hostnode **head );
void updatestruct( char *id, int x, struct node *head );
unsigned long epoch_time( void );
int mysqlquery( const char *query);
int mysqldeletelinklist( struct node *head, char *table );
int mysqldelete( char *table, char *field, char *id );
int finish_with_error( MYSQL *con );
char *my_itoa( int num, char *str );
int verify_nagios( void );
int write_configs( void );
int revert_configs( void );
int create_snapshot( void );
int restart_nagios( void );
void deleteall( struct hostnode *head );
int openfile( char *filename );
void getidForhostList( struct hostnode *head );
int check_empty_service( struct node *head, char *table );
